const express = require('express');
const faye = require('faye');
const path = require('path');

// config drone
const drone = require('ar-drone').createClient();
drone.config('general:navdata_demo', 'TRUE');

// config express app
const app = express();
app.configure(() => {
  app.set('port', process.env.PORT || 3001);
  app.use(app.router);
  app.use(express.static(path.join(__dirname, 'public')));
  app.use('/components', express.static(path.join(__dirname, 'components')));
});

// create server and socket
const server = require('http').createServer(app);
new faye.NodeAdapter({ mount: '/faye', timeout: 45 }).attach(server);
const socket = new faye.Client(`http://localhost:${app.get('port')}/faye`);

// get request from socket -> set command to drone
socket.subscribe('/drone/move', cmd => {
  console.log('move', cmd);
  if (typeof drone[cmd.action] === "function")
    return drone[cmd.action](cmd.speed);
});
socket.subscribe('/drone/animate', cmd => {
  console.log('animate', cmd);
  return drone.animate(cmd.action, cmd.duration);
});
socket.subscribe('/drone/drone', cmd => {
  console.log('drone command: ', cmd);
  if (typeof drone[cmd.action] === "function")
    return drone[cmd.action]();
});

// start server
server.listen(app.get('port'), () =>
  console.log('Express server listening on port ' + app.get('port'))
);

// send navdata to socket
drone.on('navdata', data => socket.publish('/drone/navdata', data));

// save latest img from drone PngStream
let currentImg = null;
let imageSendingPaused = false;
drone.createPngStream().on('data', frame => {
  currentImg = frame;
  if (imageSendingPaused) return;
  socket.publish('/drone/image', `/image/${Math.random()}`);
  imageSendingPaused = true;
  setTimeout(() => {
    imageSendingPaused = false;
  }, 100);
});

// send latest img to server
app.get('/image/:id', (req, res) => {
  res.writeHead(200, { 'Content-Type': 'image/png' });
  return res.end(currentImg, 'binary');
});
